"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.objectInfoToObject = exports.workspaceInfoToObject = void 0;
const json_1 = require("../../json");
const jsonLike_1 = require("../../jsonLike");
const ServiceClient_1 = require("../JSONRPC11/ServiceClient");
function workspaceInfoToObject(wsInfo) {
    return {
        id: wsInfo[0],
        name: wsInfo[1],
        owner: wsInfo[2],
        moddate: wsInfo[3],
        object_count: wsInfo[4],
        user_permission: wsInfo[5],
        globalread: wsInfo[6],
        lockstat: wsInfo[7],
        metadata: wsInfo[8],
        modDate: new Date(wsInfo[3]).toISOString(),
        modifiedAt: new Date(wsInfo[3]).getTime()
    };
}
exports.workspaceInfoToObject = workspaceInfoToObject;
function objectInfoToObject(objInfo) {
    const type = objInfo[2].split(/[-.]/);
    return {
        id: objInfo[0],
        name: objInfo[1],
        type: objInfo[2],
        save_date: objInfo[3],
        version: objInfo[4],
        saved_by: objInfo[5],
        wsid: objInfo[6],
        ws: objInfo[7],
        checksum: objInfo[8],
        size: objInfo[9],
        metadata: objInfo[10],
        ref: objInfo[6] + '/' + objInfo[0] + '/' + objInfo[4],
        obj_id: 'ws.' + objInfo[6] + '.obj.' + objInfo[0],
        typeModule: type[0],
        typeName: type[1],
        typeMajorVersion: type[2],
        typeMinorVersion: type[3],
        saveDate: new Date(objInfo[3]).toISOString(),
        savedAt: new Date(objInfo[3]).getTime()
    };
}
exports.objectInfoToObject = objectInfoToObject;
class WorkspaceClient extends ServiceClient_1.ServiceClient {
    constructor() {
        super(...arguments);
        this.module = 'Workspace';
    }
    get_object_info3(params) {
        return __awaiter(this, void 0, void 0, function* () {
            const [result] = yield this.callFunc('get_object_info3', [(0, json_1.objectToJSONObject)(params)]);
            return {
                infos: result.infos.map((info) => {
                    return objectInfoToObject(info);
                }),
                paths: result.paths
            };
        });
    }
    get_workspace_info(params) {
        return __awaiter(this, void 0, void 0, function* () {
            const [result] = yield this.callFunc('get_workspace_info', [(0, json_1.objectToJSONObject)(params)]);
            return workspaceInfoToObject(result);
        });
    }
    list_workspace_info(params) {
        return __awaiter(this, void 0, void 0, function* () {
            const [result] = yield this.callFunc('list_workspace_info', [(0, json_1.objectToJSONObject)(params)]);
            return result.map((item) => {
                return workspaceInfoToObject(item);
            });
        });
    }
    get_objects2(params) {
        return __awaiter(this, void 0, void 0, function* () {
            const [object] = yield this.callFunc('get_objects2', [(0, jsonLike_1.toJSON)(params)]);
            return object;
        });
    }
    get_permissions_mass(params) {
        return __awaiter(this, void 0, void 0, function* () {
            const [object] = yield this.callFunc('get_permissions_mass', [(0, jsonLike_1.toJSON)(params)]);
            return object;
        });
    }
    alter_workspace_metadata(params) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.callFuncEmptyResult('alter_workspace_metadata', [(0, jsonLike_1.toJSON)(params)]);
        });
    }
    ver() {
        return __awaiter(this, void 0, void 0, function* () {
            const [res] = yield this.callFuncNoParams('var');
            return res;
        });
    }
    status() {
        return __awaiter(this, void 0, void 0, function* () {
            const [res] = yield this.callFuncNoParams('status');
            return res;
        });
    }
}
exports.default = WorkspaceClient;
//# sourceMappingURL=Workspace.js.map