"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.traverse = exports.normalizePropPath = exports.objectToJSONObject = exports.isJSONValue = exports.isJSONArray = exports.isJSONObject = void 0;
;
function isJSONObject(value) {
    if (typeof value !== 'object') {
        return false;
    }
    if (value === null) {
        return false;
    }
    if (value.constructor !== {}.constructor) {
        return false;
    }
    return !Object.keys(value).some((key) => {
        return !isJSONValue(value[key]);
    });
}
exports.isJSONObject = isJSONObject;
function isJSONArray(value) {
    if (!Array.isArray(value)) {
        return false;
    }
    return !value.some((subvalue) => {
        return !isJSONValue(subvalue);
    });
}
exports.isJSONArray = isJSONArray;
function isJSONValue(value) {
    const typeOf = typeof value;
    if (['string', 'number', 'boolean'].indexOf(typeOf) >= 0) {
        return true;
    }
    if (typeof value !== 'object') {
        return false;
    }
    if (value === null) {
        return true;
    }
    if (isJSONArray(value)) {
        return true;
    }
    if (isJSONObject(value)) {
        return true;
    }
    return false;
}
exports.isJSONValue = isJSONValue;
function objectToJSONObject(obj) {
    const x = {};
    for (const [k, v] of Object.entries(obj)) {
        if (typeof k !== undefined) {
            x[k] = v;
        }
    }
    return x;
}
exports.objectToJSONObject = objectToJSONObject;
function normalizePropPath(propPath) {
    if (typeof propPath === 'string') {
        return propPath.split('.');
    }
    else if (Array.isArray(propPath)) {
        return propPath;
    }
    throw new TypeError('Invalid type for key: ' + (typeof propPath));
}
exports.normalizePropPath = normalizePropPath;
function traverse(obj, path, defaultValue) {
    const propPath = normalizePropPath(path);
    let current = obj;
    const currentPath = [];
    for (const [index, pathElement] of propPath.entries()) {
        currentPath.push(pathElement);
        if (pathElement in current) {
            const prop = current[pathElement];
            if (index === propPath.length - 1) {
                return prop;
            }
            if (isJSONObject(prop)) {
                current = prop;
            }
            else {
                throw new Error(`Cannot navigate into type "${typeof prop}" on path "${currentPath.join('.')}"`);
            }
        }
        else {
            if (typeof defaultValue === 'undefined') {
                throw new Error(`Cannot find path ${propPath.join('.')} in object`);
            }
            else {
                return defaultValue;
            }
        }
    }
    return current;
}
exports.traverse = traverse;
//# sourceMappingURL=json.js.map